@@ -1,4 +1,4 @@
-// Copyright © 2021 Rouven Spreckels <rs@qu1x.dev>
+// Copyright © 2021-2022 Rouven Spreckels <rs@qu1x.dev>
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,20 +6,18 @@
 
 // Derivative work of `core::simd` licensed under `MIT OR Apache-2.0`.
 
-use super::{Real, SimdBits, SimdMask};
+use super::{ApproxEq, Real, Select, SimdBits, SimdMask};
 use core::{
 	fmt::{Debug, LowerExp, UpperExp},
 	iter::{Product, Sum},
 	ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign},
 	ops::{Index, IndexMut},
-	simd::{LaneCount, Mask, Select, Simd, SupportedLaneCount, Swizzle, Swizzle2},
+	simd::{LaneCount, Mask, Simd, SupportedLaneCount, Swizzle, Swizzle2},
 };
 
 mod f32;
 mod f64;
 
-pub use core::simd::Which;
-
 /// Constructs vector by selecting values from the lanes of one or two source vectors.
 ///
 /// When swizzling one vector, the indices of the result vector are indicated by a `const` array of
@@ -61,10 +59,13 @@ macro_rules! swizzle {
 /// [`Simd<f64, LANES>`]: `core::simd::Simd`
 pub trait SimdReal<R: Real, const LANES: usize>
 where
+	LaneCount<LANES>: SupportedLaneCount,
 	Self: Clone + Copy + Default,
-	Self: PartialEq + PartialOrd,
+	Self: ApproxEq<R, Self> + PartialEq + PartialOrd,
 	Self: Debug + LowerExp + UpperExp,
-	Self: From<[R; LANES]> + AsRef<[R; LANES]> + AsMut<[R; LANES]>,
+	Self: From<Simd<R, LANES>> + Into<Simd<R, LANES>>,
+	Self: From<[R; LANES]> + Into<[R; LANES]>,
+	Self: AsRef<[R; LANES]> + AsMut<[R; LANES]>,
 	Self: Product<Self> + Sum<Self>,
 	for<'a> Self: Product<&'a Self> + Sum<&'a Self>,
 	Self: Index<usize, Output = R> + IndexMut<usize, Output = R>,
@@ -242,21 +243,6 @@ where
 	#[must_use]
 	fn swizzle2<T: Swizzle2<LANES, LANES>>(self, other: Self) -> Self;
 
-	/// Tests for approximate equality wrt `epsilon` or `ulp`, "or" in the sense of `||`.
-	#[must_use]
-	#[inline]
-	fn approx_eq(self, other: Self, epsilon: R, ulp: R::Bits) -> bool {
-		self.lanes_approx_eq(other, Self::splat(epsilon), Self::Bits::splat(ulp))
-			.all()
-	}
-	/// Tests for approximate inequality wrt `epsilon` or `ulp`, "and" in the sense of `&&`.
-	#[must_use]
-	#[inline]
-	fn approx_ne(self, other: Self, epsilon: R, ulp: R::Bits) -> bool {
-		self.lanes_approx_ne(other, Self::splat(epsilon), Self::Bits::splat(ulp))
-			.any()
-	}
-
 	/// Tests lanes for approximate equality wrt `epsilon` or `ulp`, "or" in the sense of `||`.
 	#[must_use]
 	fn lanes_approx_eq(self, other: Self, epsilon: Self, ulp: Self::Bits) -> Self::Mask {
@@ -362,6 +348,34 @@ where
 	#[must_use]
 	fn to_radians(self) -> Self;
 
+	/// Fused multiply-add. Computes `(self * a) + b` with only one rounding error, yielding a more
+	/// accurate result than an unfused multiply-add.
+	///
+	/// Using `mul_add` *may* be more performant than an unfused multiply-add if the target
+	/// architecture has a dedicated `fma` CPU instruction. However, this is not always true, and
+	/// will be heavily dependant on designing algorithms with specific target hardware in mind.
+	#[must_use]
+	fn mul_add(self, a: Self, b: Self) -> Self;
+	/// Produces a vector where every lane has the square root value of the equivalently-indexed
+	/// lane in `self`
+	#[must_use]
+	fn sqrt(self) -> Self;
+	/// Returns the largest integer value less than or equal to each lane.
+	#[must_use]
+	fn floor(self) -> Self;
+	/// Returns the smallest integer greater than or equal to each lane.
+	#[must_use]
+	fn ceil(self) -> Self;
+	/// Rounds to the nearest integer value. Ties round toward zero.
+	#[must_use]
+	fn round(self) -> Self;
+	/// Returns the floating point's integer value, with its fractional part removed.
+	#[must_use]
+	fn trunc(self) -> Self;
+	/// Returns the floating point's fractional value, with its integer part removed.
+	#[must_use]
+	fn fract(self) -> Self;
+
 	/// Converts an array to a SIMD vector mask.
 	#[must_use]
 	#[inline]
